/* $Id: MySQLRoleManagerDAO.java,v 1.6 2005/08/29 04:17:45 ruchida Exp $
 *
 * Copyright (c)ARGO 21, Corporation. 2005.  All rights reserved.
 * 
 * This file is part of Nautica Workflow Core.
 * 
 *  Nautica Workflow Core is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation; either version 2.1 of the License, or
 *  (at your option) any later version.
 * 
 *  Nautica Workflow Core is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 * 
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with Nautica Workflow Core Core; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *  
 */
package jp.co.argo21.nautica.workflow.dataaccess.mysql;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;

import org.apache.log4j.Logger;

import jp.co.argo21.nautica.workflow.dataaccess.AbstractDAO;
import jp.co.argo21.nautica.workflow.dataaccess.OrganizationBean;
import jp.co.argo21.nautica.workflow.dataaccess.RoleBean;
import jp.co.argo21.nautica.workflow.dataaccess.RoleManagerDAO;
import jp.co.argo21.nautica.workflow.dataaccess.UserBean;
import jp.co.argo21.nautica.workflow.engine.LogManager;

/**
 * RoleBeanf[^ƂMySQLpDAONXłB
 * <pre><tt>
 * ROLEe[u
 * +-------------+--------------+------+-----+---------+-------+
 * | Field       | Type         | Null | Key | Default | Extra |
 * +-------------+--------------+------+-----+---------+-------+
 * | ROLE_ID     | varchar(64)  |      | PRI |         |       |
 * | DESCRIPTION | varchar(128) | YES  |     |         |       |
 * +-------------+--------------+------+-----+---------+-------+
 * </tt><tt>
 * ORGROLEMAPe[u
 * +---------+-------------+------+-----+---------+-------+
 * | Field   | Type        | Null | Key | Default | Extra |
 * +---------+-------------+------+-----+---------+-------+
 * | ROLE_ID | varchar(64) |      | PRI |         |       |
 * | ORG_ID  | varchar(64) |      | PRI |         |       |
 * +---------+-------------+------+-----+---------+-------+
 * </tt><tt>
 * USERROLEMAPe[u
 * +---------+-------------+------+-----+---------+-------+
 * | Field   | Type        | Null | Key | Default | Extra |
 * +---------+-------------+------+-----+---------+-------+
 * | ROLE_ID | varchar(64) |      | PRI |         |       |
 * | USER_ID | varchar(64) |      | PRI |         |       |
 * +---------+-------------+------+-----+---------+-------+
 * </tt>
 * </pre>
 * ́AJL[lƂĂB
 * J̑啶Ɋւ炸ÃL[l͑啶Őݒ肳B
 * 
 * @author  mmanabe(Argo 21, Corp.)
 * @version $Revision: 1.6 $
 * @since   Nautica Workflow 0.9
 */
public class MySQLRoleManagerDAO extends AbstractDAO
implements RoleManagerDAO {
	/** GWO */
	static protected Logger eLog = LogManager.getEngineLogger();

	 /** [ID̃J */
    private static final String ROLE_ID = "ROLE_ID";
    
    /** l̃J */
    private static final String DESCRIPTION = "DESCRIPTION";
    
    private static final String SQL_SELECT_BY_ROLE_ID
        ="select * from ROLE where ROLE_ID = ?";
    
    private static final String SQL_SELECT_BY_ORG_ID
        = "select ROLE.* from ROLE, ORGROLEMAP" +
          " where ORGROLEMAP.ORG_ID = ? and ROLE.ROLE_ID = ORGROLEMAP.ROLE_ID;";
    
    private static final String SQL_SELECT_BY_USER_ID
        ="select ROLE.* from ROLE, USERROLEMAP" +
         " where USERROLEMAP.USER_ID = ? and ROLE.ROLE_ID = USERROLEMAP.ROLE_ID";
    
    private static final String SQL_SELECT_ORGS_BY_ROLE_ID
        = "select ORGANIZATION.* from ORGANIZATION, ORGROLEMAP" +
          " where ORGROLEMAP.ROLE_ID = ? and ORGANIZATION.ORG_ID = ORGROLEMAP.ORG_ID";
    
    private static final String SQL_SELECT_USERS_BY_ROLE_ID
        = "select USER.* from USER, USERROLEMAP" +
          " where USERROLEMAP.ROLE_ID = ? and USER.USER_ID = USERROLEMAP.USER_ID";
    
    /**
     * ֘Ae[uɑ΂ANZXIuWFNg𐶐B
     * 
     * @param conn RlNV
     */
    MySQLRoleManagerDAO(Connection conn) {
        super(conn);
    }

    /**
     * w肳ꂽIĎԂB
     * Y錠񂪑݂Ȃꍇ null ԂB
     * 
     * @param  rid ID
     * @return w肳ꂽIĎRoleBean
	 * @throws SQLException CӂSQLO
     * @see jp.co.argo21.nautica.workflow.dataaccess.RoleManagerDAO#findByRoleID(java.lang.String)
     */
    public RoleBean findByRoleID(String rid) throws SQLException {
        
        PreparedStatement st = getConnection().prepareStatement(SQL_SELECT_BY_ROLE_ID);
		try {
			st.setString(1, rid);			

			/************** DEBUG **************/
			eLog.debug(st.toString());
			/***********************************/

			ResultSet rs = st.executeQuery();
			
			RoleBean bean = null;
			if (rs.next()) {
			    bean = createBean(rs);
			}
			
			return bean;
			
		} finally {
			st.close();
		}
    }

    /**
     * w肳ꂽgĎ̔zԂB
     * Y錠񂪑݂Ȃꍇ͋̔zԂB
     * 
     * @param  oid gDID
     * @return w肳ꂽgĎRoleBean̔z
	 * @throws SQLException CӂSQLO
     * @see jp.co.argo21.nautica.workflow.dataaccess.RoleManagerDAO#findByOrganizationID(java.lang.String)
     */
    public RoleBean[] findByOrganizationID(String oid) throws SQLException {
        
        PreparedStatement st = getConnection().prepareStatement(SQL_SELECT_BY_ORG_ID);
		try {
			st.setString(1, oid);

			/************** DEBUG **************/
			eLog.debug(st.toString());
			/***********************************/

			ResultSet rs = st.executeQuery();
			
			List beanList = new ArrayList();
			while (rs.next()) {
				beanList.add(createBean(rs));
			}
						
			return (RoleBean[]) beanList.toArray(new RoleBean[beanList.size()]);
			
		} finally {
			st.close();
		}
    }

    /**
     * w肳ꂽ[Ǔ̔zԂB
     * Y錠񂪑݂Ȃꍇ͋̔zԂB
     * 
     * @param  uid [UID
     * @return w肳ꂽ[ǓRoleBean̔z
     * @throws SQLException
	 * @throws SQLException CӂSQLO
     * @see jp.co.argo21.nautica.workflow.dataaccess.RoleManagerDAO#findByUserID(java.lang.String)
     */
    public RoleBean[] findByUserID(String uid) throws SQLException {

        PreparedStatement st = getConnection().prepareStatement(SQL_SELECT_BY_USER_ID);
		try {
			st.setString(1, uid);

			/************** DEBUG **************/
			eLog.debug(st.toString());
			/***********************************/

			ResultSet rs = st.executeQuery();
			
			List beanList = new ArrayList();
			while (rs.next()) {
				beanList.add(createBean(rs));
			}
			
			return (RoleBean[]) beanList.toArray(new RoleBean[beanList.size()]);
			
		} finally {
			st.close();
		}
    }

    /**
     * w肳ꂽ蓖ĂĂgD̔zԂB
     * YgD񂪑݂Ȃꍇ͋̔zԂB
     * 
     * @param  rid ID
     * @return 蓖ĂĂgD̏OrganizationBean̔z
	 * @throws SQLException CӂSQLO
     * @see jp.co.argo21.nautica.workflow.dataaccess.RoleManagerDAO#findOrganizationsByRoleID(java.lang.String)
     */
    public OrganizationBean[] findOrganizationsByRoleID(String rid)
    throws SQLException {
        
        PreparedStatement st = getConnection().prepareStatement(SQL_SELECT_ORGS_BY_ROLE_ID);
		try {
			st.setString(1, rid);

			/************** DEBUG **************/
			eLog.debug(st.toString());
			/***********************************/

			ResultSet rs = st.executeQuery();
			
			List beanList = new ArrayList();
			while (rs.next()) {
				beanList.add(MySQLOrganizationManagerDAO.createBean(rs));
			}
						
			return (OrganizationBean[]) beanList.toArray(new OrganizationBean[beanList.size()]);
			
		} finally {
			st.close();
		}
    }

    /**
     * w肳ꂽ蓖ĂĂ郆[UԂB
     * Y郆[U񂪑݂Ȃꍇ͋̔zԂB
     * 
     * @param  rid ID
     * @return 蓖ĂĂ郆[ȔUserBean̔z
	 * @throws SQLException CӂSQLO
     * @see jp.co.argo21.nautica.workflow.dataaccess.RoleManagerDAO#findUsersByRoleID(java.lang.String)
     */
    public UserBean[] findUsersByRoleID(String rid) throws SQLException {
        
        PreparedStatement st = getConnection().prepareStatement(SQL_SELECT_USERS_BY_ROLE_ID);
		try {
			st.setString(1, rid);

			/************** DEBUG **************/
			eLog.debug(st.toString());
			/***********************************/

			ResultSet rs = st.executeQuery();
			
			List beanList = new ArrayList();
			while (rs.next()) {
				beanList.add(MySQLUserManagerDAO.createBean(rs));
			}
						
			return (UserBean[]) beanList.toArray(new UserBean[beanList.size()]);
			
		} finally {
			st.close();
		}
    }
    
    /**
     * ResultSet̏񂩂RoleBean쐬B
     * 
     * @param  rs ResultSet
     * @return ResultSet擾RoleBean
	 * @throws SQLException CӂSQLO
     */
    static RoleBean createBean(ResultSet rs) throws SQLException {

        /* J擾 */
        List columnNames = new ArrayList();
        for (int i = 1; i <= rs.getMetaData().getColumnCount(); i++) {
            columnNames.add(rs.getMetaData().getColumnName(i));
        }

        RoleBean bean = new RoleBean();
        for (int i = 0; i < columnNames.size(); i++) {
            String columnName = ((String) columnNames.get(i)).toUpperCase();
            String attribute = (String) rs.getObject(columnName);

            /* IDݒ */
            if (columnName.equals(ROLE_ID)) {
                bean.setID(attribute);
            }
            
            /* lݒ */
            if (columnName.equals(DESCRIPTION)) {
                bean.setName(attribute);                
            }

            /* IDAlȊȌɐݒ */
            if (!columnName.equals(ROLE_ID)
                    && !columnName.equals(DESCRIPTION)) {
                bean.setAttribute(columnName, attribute);
            }
        }
        return bean;
    }
}